# -*- coding: utf-8 -*-

"""
    abouttag.abouttag

    Helper test methods for the abouttag module.
    Generic entity function for simple about tags with simple normalization

    Copyright 2010 AUTHORS (see AUTHORS file)
    License: MIT, see LICENSE for more information
"""

import unittest


def generalentity(name, convention, conventions, normalize=True,
           prefix=u'', suffix=u'', case='title', doStrip=True):
    assert convention in conventions
    if normalize:
        if case == 'title':
            name = name.title()
        elif case == 'lower':
            name = name.lower()
        elif case == 'upper':
            name = name.upper()
        if doStrip:
            name = name.strip()
    return u'%s%s%s' % (prefix, name, suffix)


def simple(entity, cname=None):
    conventions = ('%s-1' % (cname if cname else entity),)

    def f(name, convention='%s-1' % entity, normalize=True):
        return generalentity(name, convention, conventions,
                             normalize=normalize, prefix=u'%s:' % entity)
    return f


class AboutTestCase(unittest.TestCase):
    def getExpectedResults(self, expected, f):
        for (input, output) in expected:
            self.assertEqual((input, f(input)), (input, output))
        self.assertEqual(f(u'/%s' % expected[0][0]) == output, False)

    def normalizeTest(self, expected, f, convention=None):
        if convention:
            for (input, normalized) in expected:
                self.assertEqual((input, f(input, convention=convention)),
                                 (input, normalized))
                self.assertEqual((input, f(input, True,
                                           convention=convention)),
                                 (input, normalized))
                self.assertEqual((input, f(input, False,
                                           convention=convention)),
                                 (input, input))
        else:
            for (input, normalized) in expected:
                self.assertEqual((input, f(input)), (input, normalized))
                self.assertEqual((input, f(input, True)), (input, normalized))
                self.assertEqual((input, f(input, False)), (input, input))

    def vectorTest(self, expected, f):
        for (args, normalized) in expected:
            self.assertEqual((args, f(*args)), (args, normalized))



        # -*- coding: utf-8 -*-

"""
    abouttag.nacolike

    NACO-like normalization.

    Copyright 2010 AUTHORS (see AUTHORS file)
    License: MIT, see LICENSE for more information
"""

import unicodedata
import re


TO_SPACES_RE = re.compile(r'[\!\(\)\{\}\<\>\-\;\:\.\?\,\/\\\@\*\%\=\$\^\_\~]')
DELETIONS_RE = re.compile(r'\[\]\|')
MULTISPACE_RE = re.compile(r'\s+')
TRANSLATE = {
        u'Æ': u'AE',
        u'æ': u'ae',
        u'Œ': u'OE',
        u'œ': u'oe',
        u'Ð': u'D',
        u'đ': u'd',
        u'ð': u'd',
        u'ı': u'i',
        u'Ł': u'l',
        u'ł': u'l',
        u'ℓ': u'l',
        u'Ø': u'o',
        u'ø': u'o',
        u'Ơ': u'o',
        u'ơ': u'o',
        u'Þ': u'th',
        u'þ': u'th',
        u'Ư': u'u',
        u'ư': u'u',
        u'α': u'a',
        u'β': u'b',
        u'γ': u'y',
        u'♭': u'b',
        u'♯': u'#',  # careful: sharp not hash on left!
        u'–': u'-',   # en-dash
        u'—': u'-',   # em-dash
        u"'": u'',    # apostrophe
}

U_TRANSLATE = {
        u'♭': u'b',
        u'♯': u'#',  # careful: sharp not hash on left!
        u'–': u'-',   # en-dash
        u'—': u'-',   # em-dash
        u"'": u'',    # apostrophe
}


def remove_accents(u):
    return ''.join(c for c in unicodedata.normalize('NFD', u)
                            if unicodedata.category(c) != 'Mn') if u else u


def normalize_part(u, preserveAlpha=False):
    if preserveAlpha:
        u = ''.join(U_TRANSLATE[s] if s in U_TRANSLATE else s for s in u)
    else:
        u = ''.join(TRANSLATE[s] if s in TRANSLATE else s for s in u)
        u = remove_accents(u)
    u = re.sub(TO_SPACES_RE, u' ', u)
    u = re.sub(DELETIONS_RE, u'', u)
    if preserveAlpha:
        u = ''.join(c if (0x1F < ord(c) < 0x7E or c.isalpha())
                      else u' ' for c in u)
    else:
        u = ''.join(c if 0x1F < ord(c) < 0x7E else u' ' for c in u)
    u = re.sub(MULTISPACE_RE, u' ', u)
    return u


def normalize(u, preserveFirstComma=False, preserveAlpha=False):
    """Normalize a string using NACO normalization rules.
       Input is (must be) unicode string.

       Rules taken from NACO Normalization
       http://lcweb.loc.gov/catdir/pcc/naco/normrule.html
       Except that accents are mapped to nearest equivalent ASCII characters
       or character sequences, and "subfield delimiters" are ignored.
    """
    if preserveFirstComma and u',' in u:
        cPos = u.find(u',')
        U = u'%s,%s' % (normalize_part(u[:cPos], preserveAlpha),
                        normalize_part(u[:cPos]), preserveAlpha)
    else:
        U = normalize_part(u, preserveAlpha)
    return U.lower().strip()



# -*- coding: utf-8 -*-

"""
    abouttag.film

    Standard about tags for films (movies)

    Copyright 2010 AUTHORS (see AUTHORS file)
    License: MIT, see LICENSE for more information
"""

import unittest
from abouttag import about
from abouttag.nacolike import normalize
from abouttag.books import normalize_work

normalize_film = normalize_work(u'film', preserveAlpha=True,
                                move_surname=False)


def film(title, year, **kwargs):
    """Usage:
        from abouttag.film import film

        print film(u"Citizen Kane", u'1941')
    """
    if 'convention' in kwargs:
        assert kwargs['convention'].lower() == u'track-u'
    if type(year) == int:
        year = str(year)
    assert len(year) == 4
    assert all('0' <= y <= '9' for y in year)

    return normalize_film(title, year, **kwargs)

movie = film


class TestFilm(about.AboutTestCase):
    def testFluidDBNormalizeFilm(self):
        expected = (
            ((u"Citizen Kane", u'1941'), u'film:citizen kane (1941)'),
            ((u"The Last Seduction", 1994), u'film:the last seduction (1994)'),
            ((u"Él", 1953), u'film:él (1953)'),
            ((u"L'Âge d'Or", u'1930'), u'film:lâge dor (1930)'),
        )
        bads = ((u"Citizen Kane", u''),
                (u"Citizen Kane", u'1941-42'),
                (u"Citizen Kane", u'41')
        )
        for (input, output) in expected:
            title, year = input[0], input[1]
            self.assertEqual((input, output), (input, film(title, year)))
            self.assertEqual((input, output), (input, movie(title, year)))
        for input in bads:
            title, year = input[0], input[1]
            self.assertRaises(AssertionError, film, title, year)


                             

if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-

"""
    abouttag.test

    Run tests for the whole abouttag module.

    Copyright 2010 AUTHORS (see AUTHORS file)
    License: MIT, see LICENSE for more information
"""

import unittest
from about import *
from abouttag.fluiddb import *
from abouttag.uri import *
from abouttag.database import *
from abouttag.objects import *
from abouttag.location import *
from abouttag.books import *
from abouttag.music import *
from abouttag.film import *

unittest.main()






from amazon import get_about_tag_for_item

print get_about_tag_for_item('http://www.amazon.com/Renbourn-Stefan-Grossman-Live-Concert/dp/B000000E8R/ref=sr_1_7?ie=UTF8&qid=1308066155&sr=8-7')

print get_about_tag_for_item('http://www.amazon.com/gp/product/B0027J9SM8/ref=dm_mu_dp_trk1')





# -*- coding: utf-8 -*-

"""
    abouttag.database

    Standard about tags for database tables / datasets

    Copyright 2010 AUTHORS (see AUTHORS file)
    License: MIT, see LICENSE for more information
"""

from abouttag import about


class Database:
    """Usage:
        from abouttag.database import Database

        db = Database()
        table = db.table(u'elements')
        field = db.field(u'Name', u'elements')
    """
    def __init__(self, convention=u'database-1'):
        assert convention.lower() in (u'database-1',)
        self.convention = convention.lower()

    def normalize(self, word, normalize=True):
        return word.strip()

    def table(self, tablename, normalize=True):
        return u'table:%s' % self.normalize(tablename)

    def field(self, fieldname, tablename, normalize=True):
        return u'field:%s in table:%s' % (self.normalize(fieldname),
                                          self.normalize(tablename))


class TestDatabase(about.AboutTestCase):
    def testDatabseBadConvention(self):
        self.assertRaises(AssertionError, Database, u'unknown')

    def testTable(self):
        db = Database()
        self.assertEqual(db.table(u'elements'), u'table:elements')

    def testFluidDBNamespace(self):
        db = Database()
        self.assertEqual(db.field(u'Name', u'elements'),
                         u'field:Name in table:elements')



# -*- coding: utf-8 -*-

"""
    abouttag.generic

    Provides generic abouttag function; object type specified by first
    parameter.

    Copyright 2010 AUTHORS (see AUTHORS file)
    License: MIT, see LICENSE for more information
"""

from abouttag.fluiddb import FluidDB
from abouttag.uri import URI
from abouttag.database import Database
from abouttag.objects import planet, element, twitteruser
from abouttag.location import GEOnet
from abouttag.books import book, ubook, author
from abouttag.music import album, track, artist, isrc_recording

class AboutTagError(Exception):
    pass


def abouttag(*args):

    items = {
        'fi-user': FluidDB.user,
        'fi-namespace': FluidDB.namespace,
        'fi-ns': FluidDB.namespace,
        'fi-tag': FluidDB.tag,
        'uri': URI,
        'url': URI,
        'URI': URI,
        'URL': URI,
        'db-table': Database.table,
        'db-field': Database.field,
        'planet': planet,
        'element': element,
        'book': ubook,
        'author': author,
        'album': album,
        'track': track,
        'artist': artist,
        'isrc-recording': isrc_recording,
        'twitter-user': twitteruser,
    }
             
    if len(args) == 0:
        raise AboutTagError('No object type specified')
    object = args[0]
    if not object in items:
        raise AboutTagError('Object type %s not known.\n'
                            '  Supported object types are:\n    %s\n'
                            % (object, '\n    '.join(items.keys())))

    f = items[object]
    if object.startswith('fi'):
        db = FluidDB()
        return f(db, *args[1:])
    elif object.startswith('db'):
        db = Database()
        return f(db, *args[1:])
    elif object == 'author':
        if len(args) > 2:
            return f(args[1], *(int(a) for a in args[2:]))
        else:
            raise AboutTagError('No author specified')
    else:
        return f(*args[1:])






# -*- coding: utf-8 -*-

"""
    abouttag.objects

    Objects with very simple conventions, mostly
        "entity:Name"
    with normalization consisting only stripping leading
    and trailing spaces and (perhaps) normalizing it.

    Copyright 2010 AUTHORS (see AUTHORS file)
    License: MIT, see LICENSE for more information
"""
from abouttag import about

planet = about.simple(u'planet')
element = about.simple(u'element')
twitteruser1 = about.simple(u'twitter.com:uid', 'twitteruser')

def twitteruser(name, normalize=True, convention=u'twitteruser-2'):
    assert convention in (u'twitteruser-1', u'twitteruser-2')
    if convention == u'twitteruser-1':
        return twitteruser1(name, convention, normalize)
    else:
        n = name.lower().strip() if normalize else name
        return n if n.startswith(u'@') else u'@%s' % n


class TestObjects(about.AboutTestCase):
    # Just tests planet, becaues if planet's OK there's really
    # not much to go wrong with other simple about tags.
    def testConvention(self):
        self.assertRaises(AssertionError, planet, u'foo', u'unknown')

    def testPlanet(self):
        self.assertEqual(planet(u'Mars'), u'planet:Mars')
        self.assertEqual(planet(u' saturn '), u'planet:Saturn')

        # No normalization:
        self.assertEqual(planet(u' saturn ', normalize=False),
                         u'planet: saturn ')

    def testElement(self):
        self.assertEqual(element(u'Hydrogen'), u'element:Hydrogen')
        self.assertEqual(element(u' copper '), u'element:Copper')

        # No normalization:
        self.assertEqual(element(u' copper ', normalize=False),
                         u'element: copper ')

    def testTwitterUser(self):
        self.assertEqual(twitteruser(u'17895882', convention='twitteruser-1'),
                         u'twitter.com:uid:17895882')
        self.assertEqual(twitteruser(u' 17895882 ',
                                     convention='twitteruser-1'),
                         u'twitter.com:uid:17895882')

        self.assertEqual(twitteruser(u'terrycojones'), u'@terrycojones')
        self.assertEqual(twitteruser(u' @TERRYCOJONES '), u'@terrycojones')
                         

        # No normalization:
        self.assertEqual(twitteruser(u' @TERRYCOJONES ', False,
                                     'twitteruser-2'),
                         u'@ @TERRYCOJONES ')



# -*- coding: utf-8 -*-

"""
    abouttag.fluiddb

    Standard about tags as used by FluidDB itself.

    Copyright 2010 AUTHORS (see AUTHORS file)
    License: MIT, see LICENSE for more information
"""

from abouttag import about


class FluidDB:
    """Usage:
        from abouttag.fluiddb import FluidDB

        db = FluidDB()
        user = db.user(u'njr')
        ns = db.namespace(u'njr/ideas')
        tag = db.tag(u'njr/rating')
    """
    def __init__(self, convention=u'fluiddb-1'):
        assert convention.lower() in (u'fluiddb-1',)
        self.convention = convention.lower()

    def normalize(self, path, normalize=True, convention=u'fluiddb-1'):
        if normalize:
            if path.startswith(u'/'):
                path = path[1:]
            if path.endswith(u'/'):
                path = path[:-1]
            path = path.strip()
        return path

    def user(self, username, normalize=True):
        # There is no normalization to perform on a username
        return u'Object for the user named %s' % username

    def namespace(self, ns, normalize=True):
        return u'Object for the namespace %s' % self.normalize(ns, normalize)

    def tag(self, tag, normalize=True):
        return 'Object for the attribute %s' % self.normalize(tag, normalize)


class TestFluidDB(about.AboutTestCase):
    def testFluidDBBadConvention(self):
        self.assertRaises(AssertionError, FluidDB, u'unknown')

    def testFluidDBUser(self):
        db = FluidDB(u'fluiddb-1')
        self.assertEqual(db.user(u'njr'), u'Object for the user named njr')

    def testFluidDBNormalize(self):
        expected = (
            (u'njr', u'njr'),
            (u'/njr', u'njr'),
            (u'njr/', u'njr'),
            (u'/njr/', u'njr'),
            (u'one/two', u'one/two'),
        )
        db = FluidDB()
        self.normalizeTest(expected, db.normalize)

    def testFluidDBNamespace(self):
        expected = (
            (u'njr', u'Object for the namespace njr'),
            (u'one/two', u'Object for the namespace one/two')
        )
        db = FluidDB()
        self.getExpectedResults(expected, db.namespace)

    def testFluidDBTag(self):
        expected = (
            (u'njr/rating', u'Object for the attribute njr/rating'),
            (u'/njr/sub/rating', u'Object for the attribute njr/sub/rating'),
        )
        db = FluidDB()
        self.getExpectedResults(expected, db.tag)



# -*- coding: utf-8 -*-
import re
import unittest

import urllib
import books
import music

AZ_NO_RE = r'^(.*)(\(\d{6}\d*\))$'
AZ_EDITION_RE = r'^(.*)(\(.* edition\))$'
ENCODING = 'latin-1'

class NoTitleError(Exception):
    pass


class NotBookError(Exception):
    pass


def get_html_page_title(url):
    """Gets the stripped contents of the <title> element from a web page,
       specified by a (unicode) URL.

    """
    h = urllib.urlopen(url.encode('UTF-8'))
    line = h.readline()
    while not ('<title') in line:
        line = h.readline()
    lines = [line]
    while not ('</title') in line:
        line = h.readline()
        line.append(line)

    title = ' '.join(lines)
    start = title.find('<title')
    end = title.find('</title')
    title = title[start:end]
    start = title.find('>')
    if start >= 0:
        title = title[start+1:]
    else:
        raise NoTitleError(u'The URL %s apparently has no title' % url)
    return title.replace('&#39;', "'"), h


def process_title(title, kinds):
    parts = [p.strip() for p in title.split(':')]
    kind = parts[-1].lower()
    if not (len(parts) >= 4 and kind in kinds):
        return None, None, None
    parts = parts[:-1]
    for i in range(len(parts)):
        if parts[i].lower().startswith('amazon.'):
            del parts[i]
            break
    if not len(parts) > 1:
        return None, None, None
    title, work = ': '.join(parts[:-1]), parts[-1]
    return title, work, kind


def get_about_tag_for_book(title, h):
    """Attempts to find the information about a product from
       an Amazon UK or Amazon US web page.
       Currently suports only books (and e-books).

       If it succeeds, this returns the best guess at the book-u
       about tag for that book.

       If not, returns None.
    """
    title, author, kind = process_title(title, ('books', 'kindle store'))
    if kind is None:
        return None
    authors = author.decode(ENCODING).split(u',')
    m = re.match(AZ_NO_RE, title)
    if m:
        title = m.group(1).strip()
    m = re.match(AZ_EDITION_RE, title.lower())
    if m:
        title = title[:-len(m.group(2))].strip()
    if title.lower().endswith('ebook'):
        title = title[:-5].strip()
    title = title.decode(ENCODING)
        
    return books.ubook(title, *authors)

def get_about_tag_for_music(title, h):
    work, artist, kind = process_title(title, ('music', 'mp3 downloads'))
    if kind is None:
        return None
    a = artist
    i = a.find(',')     # Change final comma to ampersand
    if i >= 0:
        while i > 0:
            a = a[i+1:]
            i = a.find(',')
        assert artist[-len(a) - 1] == ','
        artist = artist[:-len(a)] + '&' + artist[-len(a):]
    if kind == 'music':
        return music.album(work.decode(ENCODING),  artist.decode(ENCODING))
    else:
        return music.track(work.decode(ENCODING),  artist.decode(ENCODING))

def get_about_tag_for_item(url):
    item = None
    title, h = get_html_page_title(url)
    for method in (get_about_tag_for_book,
                   get_about_tag_for_music):
        item = method(title, h)
        if item:
            break
            h.close()
    return item


class TestAmazonBooks(unittest.TestCase):
    def testURLs(self):
        bookurls = (
            (u"http://www.amazon.co.uk/gp/product/0321616952/ref=s9_simh_gw_p14_d2_i1?pf_rd_m=A3P5ROKL5A1OLE&pf_rd_s=center-2&pf_rd_r=18GX16Y6EBQVN4SWE9SZ&pf_rd_t=101&pf_rd_p=467128533&pf_rd_i=468294", u"book:designing with web standards voices that matter (jeffrey zeldman; ethan marcotte)"),

        (u"http://www.amazon.com/gp/product/0321616952/ref=s9_simh_gw_p14_d2_i1?pf_rd_m=ATVPDKIKX0DER&pf_rd_s=center-2&pf_rd_r=1QR600K59G2T86FF5KXE&pf_rd_t=101&pf_rd_p=470938631&pf_rd_i=507846", u'book:designing with web standards (jeffrey zeldman; ethan marcotte)'),

       (u"http://www.amazon.com/Before-I-Go-Sleep-Novel/dp/0062060554/ref=bhp_9p2_botm_02?pf_rd_m=ATVPDKIKX0DER&pf_rd_s=center-4&pf_rd_r=1MW9TP9FWZJZ4DXAXSES&pf_rd_t=101&pf_rd_p=1299446602&pf_rd_i=283155", u'book:before i go to sleep a novel (s j watson)'),

        (u"http://www.amazon.com/Hundred-Solitude-Gabriel-Garcia-Marquez/dp/0060929790/ref=sr_1_1?s=books&ie=UTF8&qid=1307974869&sr=1-1", 'book:one hundred years of solitude (gabriel garcia marquez)'),

        (u"http://www.amazon.co.uk/Hundred-Solitude-Gabriel-Garcia-Marquez/dp/014103243X/ref=sr_1_1?ie=UTF8&qid=1307974975&sr=8-1", u'book:one hundred years of solitude (gabriel garcia marquez)'),

        (u"http://www.amazon.co.uk/Café-Best-Coffee-Shop-Design/dp/3037680458/ref=sr_1_3?ie=UTF8&qid=1307975275&sr=8-3", u'book:café best of coffee shop design (braun)'),

        (u"http://www.amazon.com/The-Café-Book-ebook/dp/B004ZZUNPI/ref=sr_1_15?ie=UTF8&qid=1307975833&sr=8-15", u'book:the café book (gail boushey; joan moser)'),
        (u"http://www.amazon.co.uk/The-Lacuna-ebook/dp/B002TVSF9Q/ref=sr_1_1?ie=UTF8&m=A3TVV12T0I6NSM&s=digital-text&qid=1307976072&sr=1-1", u'book:the lacuna (barbara kingsolver)'),
        )

        for (url, abouttag) in bookurls:
            self.assertEqual(get_about_tag_for_item(url), abouttag)

    def testNonBookURL(self):
        url = u"http://www.amazon.co.uk/Kindle-Wireless-Reader-Wifi-Graphite/dp/B002Y27P46/ref=amb_link_160235127_2?pf_rd_m=A3P5ROKL5A1OLE&pf_rd_s=center-1&pf_rd_r=0Z9JKC2J7ERVSXFD39XS&pf_rd_t=101&pf_rd_p=244141367&pf_rd_i=468294"

        self.assertEqual(get_about_tag_for_item(url), None)

    def testNonAmazonURL(self):
        url = u"http://stochasticsolutions.com"
        self.assertEqual(get_about_tag_for_item(url), None)

if __name__ == '__main__':
    unittest.main()




# -*- coding: utf-8 -*-

"""
    abouttag.books

    Standard about tags for books and authors.

    Copyright 2010 AUTHORS (see AUTHORS file)
    License: MIT, see LICENSE for more information
"""

import unittest
import re
from abouttag import about
from abouttag.nacolike import normalize

DOT_LETTER_RE = re.compile(u'^.*\.[A-Z].*$')
LC_ARTICLES = (u'the', u'a')

def replacedots(author):
    return author.replace(u'.', u' ')


def book_author_about(author, move_surname=True):
    if move_surname:
        return replacedots(move_surname_to_end(author))
    else:
        return replacedots(author)


def normalize_work(prefix, preserveAlpha=False, move_surname=True):
    def f(title, *authors, **kwargs):
        doNormalize = kwargs['normalize'] if 'normalize' in kwargs else True
        if doNormalize:
            authors = u'; '.join(normalize(book_author_about(a, move_surname),
                                           preserveAlpha=preserveAlpha)
                                  for a in authors if a)
            return u'%s:%s (%s)' % (prefix,
                                    normalize(move_article(title),
                                              preserveAlpha=preserveAlpha),
                                    authors)
        else:
            authors = u'; '.join(a for a in authors if a)
            return u'%s:%s (%s)' % (prefix, title, authors)
    return f


normalize_book = normalize_work(u'book')
normalize_ubook = normalize_work(u'book', preserveAlpha=True)


def book(title, *authors, **kwargs):
    """This uses the book-u convention which is the same as book-1
except that accents are preserved.

Usage:
        from abouttag.books import ubook

        print book(u"Gödel, Escher, Bach: An Eternal Golden Braid",
                   u'Douglas R. Hofstader')
        book:gödel escher bach an eternal golden braid (douglas r hofstader)


        print book(u'The Feynman Lectures on Physics',
                   u'Richard P. Feynman', u'Robert B. Leighton',
                   u'Matthew Sands')
        book:the feynman lectures on physics (richard p feynman; robert b leighton; matthew sands)


        print book(u'The Oxford English Dictionary: second edition, volume 3',
                   u'John Simpson', u'Edmund Weiner')

        book:the oxford english dictionary second edition volume 3 (john simpson; edmund weiner)
    """
    if 'convention' in kwargs:
        assert kwargs['convention'].lower() == u'book-u'

    return normalize_ubook(title, *authors, **kwargs)


ubook = book


def abook(title, *authors, **kwargs):
    """Usage:
        from abouttag.books import abook

        print abook(u"Gödel, Escher, Bach: An Eternal Golden Braid",
                   u'Douglas R. Hofstader')
        book:godel escher bach an eternal golden braid (douglas r hofstader)


        print abook(u'The Feynman Lectures on Physics',
                    u'Richard P. Feynman', u'Robert B. Leighton',
                    u'Matthew Sands')
        book:the feynman lectures on physics (richard p feynman; robert b leighton; matthew sands)


        print abook(u'The Oxford English Dictionary: second edition, volume 3',
                    u'John Simpson', u'Edmund Weiner')

        book:the oxford english dictionary second edition volume 3 (john simpson; edmund weiner)
    """
    if 'convention' in kwargs:
        assert kwargs['convention'].lower() == u'book-1'

    return normalize_book(title, *authors, **kwargs)


def is_all_upper(s):
    """Returns True if the whole of the string s is upper case."""
    return sum(c.isupper() for c in s) == len(s)


def dot_initial(s):
    if len(s) == 1 and s.isupper():
        return u'%s.' % s
    else:
        return s


def move_article(title):
    """Moves trailing article after comma to the start of a title.
       So

          Catcher in the Rye, The --> The Catcher in the Rye
          Catcher in the Rye,The  --> The Catcher in the Rye
          The Catcher in the Rye  --> The Catcher in the Rye
          Catcher in the Rye      --> Catcher in the Rye
          A Stitch in Time        --> A Stitch in Time
          Stitch in Time, A       --> A Stitch in Time
          Stitch in Time,A        --> A Stitch in Time
          Stitch in Time          --> Stitch in Time

       Currently only handles 'a' and 'the', but it just uses
       a word list, so it would be easy to add le/la etc.

       Case is unaffected.
        
    """
    L = title.lower()
    if not title:
        return u''
    for a in LC_ARTICLES:
        if L.endswith(u', ' + a):
            return u'%s %s' % (title[-len(a):], title[:-(len(a) + 2)])
        elif L.endswith(u',' + a):
            return u'%s %s' % (title[-len(a):], title[:-(len(a) + 1)])
    return title


def move_surname_to_end(author):
    """Given a name such as "Salinger, J.D.", this functions transforms
       the name into J. D. Salinger.

       Specifically, it takes anything after the comma (if present)
       and moves it to the front, adds full stops after single initials,
       and splits initials.   So all of

        J. D. Salinger
        J.D.Salinger
        J.D. Salinger
        JD Salinger
        Salinger,J.D.
        Salinger, J.D.
        Salinger, J. D.
        Salinger, JD

        will be turned into u'J. D. Salinger'.
        
    """
    if not author:
        return u''

    parts = [s.strip() for s in author.split(u',')]
    if len(parts) >= 2:
        parts = parts[1:] + [parts[0]]
        author = u' '.join(parts)

    parts = [s.strip() for s in author.split(u' ')]
    if len(parts) < 1 or is_all_upper(parts[-1]):
        return author           # Surname thing seems to be upper case;
                                # don't try to split initials.

    if len(parts[0]) == 1:       # First part is undotted initial
        return u' '.join([dot_initial(p) for p in parts[:-1]] + [parts[-1]])
    elif is_all_upper(parts[0]):    # First part looks like string of undotted
                                    # initials
        return u' '.join([u'%s.' % p for p in parts[0]] + parts[1:])
    elif re.match(DOT_LETTER_RE, parts[0]):
        parts[0] =  u'. '.join(s.strip() for s in parts[0].split(u'.')).strip()
        return u' '.join(parts)
    else:
        return author


def normalize_author(author, year, month, day, **kwargs):
    doNormalize = kwargs['normalize'] if 'normalize' in kwargs else True
    if year:
        if month:
            if day:
                date = u' (%04d-%02d-%02d)' % (year, month, day)
            else:
                date = u' (%04d-%02d)' % (year, month)
        else:
            date = u' (%04d)' % (year)
    else:
        date = u''
    if doNormalize:
        author = normalize(replacedots(author))
    return u'author:%s%s' % (author, date)


def author(name, year=None, month=None, day=None, **kwargs):
    """Usage:
        from abouttag.books import author

        print author(u'Douglas R. Hofstader', 1945, 2, 15)
        author:douglas r hofstader (1945-02-15)

        print author(u'Douglas R. Hofstader')
        author:douglas r hofstader

    """
    if 'convention' in kwargs:
        assert kwargs['convention'].lower() == u'author-1'

    return normalize_author(name, year, month, day, **kwargs)


class TestBooks(about.AboutTestCase):
    def testFluidDBBadConvention(self):
        self.assertRaises(AssertionError, book, u'hello', convention='unknown')

    def testFluidDBNormalize(self):
        expected = (
            ((u"Gödel, Escher, Bach: An Eternal Golden Braid",
              u'Douglas R. Hofstadter'),
             u'book:godel escher bach an eternal golden braid '
                 u'(douglas r hofstadter)'),

             ((u'The Feynman Lectures on Physics',
               u'Richard P. Feynman', u'Robert B. Leighton',
               u'Matthew Sands'),
              u'book:the feynman lectures on physics '
                  u'(richard p feynman; robert b leighton; matthew sands)'),

             ((u'The Feynman Lectures on Physics',
               u'Richard P. Feynman', u'', u'Robert B. Leighton', None,
               u'Matthew Sands'),
              u'book:the feynman lectures on physics '
                  u'(richard p feynman; robert b leighton; matthew sands)'),

             ((u'The Oxford English Dictionary: second edition, volume 3',
               u'John Simpson', u'Edmund Weiner'),
              u'book:the oxford english dictionary second edition volume 3 '
                  '(john simpson; edmund weiner)'),
        )
        for (input, output) in expected:
            title, author = input[0], input[1:]
            self.assertEqual((input, output),
                             (input, abook(title, *author)))

    def testFluidDBNormalizeU(self):
        expected = (
            ((u"Gödel, Escher, Bach: An Eternal Golden Braid",
              u'Douglas R. Hofstader'),
             u'book:gödel escher bach an eternal golden braid '
                 u'(douglas r hofstader)'),

             ((u'The Feynman Lectures on Physics',
               u'Richard P. Feynman', u'Robert B. Leighton',
               u'Matthew Sands'),
              u'book:the feynman lectures on physics '
                  u'(richard p feynman; robert b leighton; matthew sands)'),

             ((u'The Feynman Lectures on Physics',
               u'Richard P. Feynman', u'', u'Robert B. Leighton', None,
               u'Matthew Sands'),
              u'book:the feynman lectures on physics '
                  u'(richard p feynman; robert b leighton; matthew sands)'),

             ((u'The Oxford English Dictionary: second edition, volume 3',
               u'John Simpson', u'Edmund Weiner'),
              u'book:the oxford english dictionary second edition volume 3 '
                  '(john simpson; edmund weiner)'),

            ((u'The One Hundred Years of Solitude',
              u'Gabriel García Márquez'),
             u'book:the one hundred years of solitude '
             u'(gabriel garcía márquez)'),

            ((u"L'Histoire de Gil Blas de Santillane",
              u'Alain-René Lesage'), 
             u'book:lhistoire de gil blas de santillane (alain rené lesage)'),

           ((u'Cønsümàte Craziness',
             u'Imprøbable', u'CrÆzy', u'Øutlandisch'),
            u'book:cønsümàte craziness (imprøbable; cræzy; øutlandisch)'),
        )
        for (input, output) in expected:
            title, author = input[0], input[1:]
            self.assertEqual((input, output),
                             (input, book(title, *author)))
            self.assertEqual((input, output),
                             (input, ubook(title, *author)))

    def testNormalizeAuthor(self):
        expected = (
            ((u"Douglas R. Hofstadter", 1945, 2, 15),
             u'author:douglas r hofstadter (1945-02-15)'),
            ((u"Douglas R. Hofstadter", 1945, 2, None),
             u'author:douglas r hofstadter (1945-02)'),
            ((u"Douglas R. Hofstadter", 1945, None, None),
             u'author:douglas r hofstadter (1945)'),
            ((u"Douglas R. Hofstadter", None, None, None),
             u'author:douglas r hofstadter'),
            ((u'Gabriel García Márquez', 1927, 3, 6),
             u'author:gabriel garcia marquez (1927-03-06)'),
        )
        for (input, output) in expected:
            [name, y, m, d] = input
            self.assertEqual((input, output),
                             (input, author(name, y, m, d)))


    def testMoveSurname1(self):
        inputs = [
            u'J. D. Salinger',
            u'J.D.Salinger',
            u'J.D. Salinger',
            u'JD Salinger',
            u'Salinger,J.D.',
            u'Salinger, J.D.',
            u'Salinger, J. D.',
            u'Salinger, JD',
        ]
        for input in inputs:
            self.assertEqual((input, move_surname_to_end(input)),
                             (input, u'J. D. Salinger'))


    def testMoveSurname2(self):
        inputs = [
            u'Anne Michaels',
            u'Michaels, Anne',
            u'Michaels,Anne',
        ]
        for input in inputs:
            self.assertEqual((input, move_surname_to_end(input)),
                             (input, u'Anne Michaels'))


    def testMoveSurname3(self):
        inputs = [
            u'Lynne Reid Banks',
            u'Banks, Lynne Reid',
            u'Banks,Lynne Reid',
            u'Reid Banks, Lynne',
            u'Reid Banks,Lynne',
        ]
        for input in inputs:
            self.assertEqual((input, move_surname_to_end(input)),
                             (input, u'Lynne Reid Banks'))

    def testMoveArticles(self):
        IO = ((u'Catcher in the Rye, The', u'The Catcher in the Rye'),
              (u'Catcher in the Rye,The', u'The Catcher in the Rye'),
              (u'The Catcher in the Rye', u'The Catcher in the Rye'),
              (u'Catcher in the Rye', u'Catcher in the Rye'),
              (u'A Stitch in Time', u'A Stitch in Time'),
              (u'Stitch in Time, A', u'A Stitch in Time'),
              (u'Stitch in Time,A', u'A Stitch in Time'),
              (u'Stitch in Time', u'Stitch in Time'))
        for (input, output) in IO:
            self.assertEqual((input, move_article(input)),
                             (input, output))


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
from distutils.core import setup

setup(name='abouttag',
      version='1.1',
      description='Normalizes about tags for Fluidinfo',
      author='Nicholas J. Radcliffe',
      author_email='njr@StochasticSolutions.com',
      packages=['abouttag'],
      url="http://github.com/njr0/abouttag/",
      download_url="https://github.com/njr0/abouttag",
      keywords=[
          'fluidinfo',
          'about',
          'tag',
          'normalization',
          'canonicalization',
          'standardarization'
      ],
      classifiers = [
          'Programming Language :: Python',
          'Programming Language :: Python :: 2',
          'Development Status :: 5 - Production/Stable',
          'License :: OSI Approved :: MIT License',
          'Environment :: Other Environment',
          'Intended Audience :: Developers',
          'Topic :: Software Development :: Libraries :: Python Modules',
          'Topic :: Internet',
      ],
      install_requires = ['urlnorm>=1.1.2'],
      long_description = '''\
This package provides functions for generating about tags for
Fluidinfo following various conventions.

Fluidinfo is a hosted, online database based on the notion of tagging.
For more information on FluidDB, visit http://fluidinfo.com.

For more information on the ideas that motivated this module,
see posts at http://abouttag.blogspot.com.   A good place to
start is

http://abouttag.blogspot.com/2010/03/about-tag-conventions-in-fluiddb.html

Examples
--------
::

    from abouttag.books import book, ubook, author
    from abouttag.music import album, track, artist, isrc_recording
    from abouttag.film import film, movie

    print '\nBOOKS (book-u convention):\n'

    print book(u"Gödel, Escher, Bach: An Eternal Golden Braid",
               u'Douglas R. Hofstader')
    print
    print book(u"One Hundred Years of Solitude", u'Gabriel García Márquez')
    print book(u'The Feynman Lectures on Physics',
               u'Richard P. Feynman', u'Robert B. Leighton',
               u'Matthew Sands')
    print
    print book(u'The Oxford English Dictionary: second edition, volume 3',
               u'John Simpson', u'Edmund Weiner')
    print

    print '\nAUTHORS:'

    print author(u'Gabriel García Márquez', 1927, 3, 6)
    print author(u"Douglas R. Hofstadter", 1945, 2, 15)

    print '\nMUSIC (album-u, track-u, artist-u and isrc-1 conventions):\n'
    print track(u'Bamboulé', u'Bensusan and Malherbe')
    print album(u"Solilaï", u'Pierre Bensusan')
    print artist(u"Crosby, Stills, Nash & Young")
    print isrc_recording(u'US-PR3-73-00012')

    print '\nFILM (film-u convention):\n'
    print film(u"Citizen Kane", u'1941')
    print movie(u"L'Âge d'Or", u'1930')
    print


Installation
------------

::

    pip install -U abouttag


Dependencies
------------

`urlnorm <http://pypi.python.org/pypi/urlnorm>`_


      '''
      
     )



# -*- coding: utf-8 -*-

"""
    abouttag.uri

    Standard about tags for URIs and URLs

    Copyright 2010 AUTHORS (see AUTHORS file)
    License: MIT, see LICENSE for more information
"""

from abouttag import about
import urlnorm


def normalize_uri_1(uri, normalize=True):
    if normalize:
        if not u'://' in uri:
            uri = u'http://%s' % uri
        if uri.endswith(u'/'):
            uri = uri[:-1]
        parts = uri.split(u'://')
        prefix, path = parts[0].lower(), u'://'.join(parts[1:])
        parts = path.split(u'/')
        domain, rest = parts[0].lower(), u'/'.join(parts[1:])
        if rest:
            uri = u'%s://%s/%s' % (prefix, domain, rest)
        else:
            uri = u'%s://%s' % (prefix, domain)
        uri = uri.strip()
    return uri


def normalize_uri(uri, normalize=True):
    if normalize:
        uri = urlnorm.norm(uri)
    return uri


def URI(uri, normalize=True, convention=u'uri-2'):
    """Usage:
        from abouttag.uri import URI

        normalURL = URI(u'FluidDB.fluidinfo.com')
    """
    assert convention.lower() in (u'uri-1', u'uri-2')
    if convention == u'uri-1':
        return normalize_uri_1(uri, normalize)
    else:
        if normalize and not u'://' in uri:
            uri = u'http://' + uri
        return normalize_uri(uri, normalize)


class TestURI(about.AboutTestCase):
    def testFluidDBBadConvention(self):
        self.assertRaises(AssertionError, URI, u'hello', convention='unknown')

    def testFluidDBNormalize1(self):
        expected = (
            ('FluidDB.fluidinfo.com', 'http://fluiddb.fluidinfo.com'),
            ('FluidDB.fluidinfo.com/one/two/',
                'http://fluiddb.fluidinfo.com/one/two'),
            ('https://FluidDB.fluidinfo.com/one/two/',
                'https://fluiddb.fluidinfo.com/one/two'),
            ('http://fluiddb.fluidinfo.com/one/two/',
                'http://fluiddb.fluidinfo.com/one/two'),
            ('http://test.com/one/two/?referrer=http://a.b/c',
                'http://test.com/one/two/?referrer=http://a.b/c')
        )
        self.normalizeTest(expected, URI, convention=u'uri-1')

    def testFluidDBNormalize(self):
        expected = (
            ('FluidDB.fluidinfo.com', 'http://fluiddb.fluidinfo.com/'),
            ('FluidDB.fluidinfo.com/one/two/',
                'http://fluiddb.fluidinfo.com/one/two/'),
            ('https://FluidDB.fluidinfo.com/one/two/',
                'https://fluiddb.fluidinfo.com/one/two/'),
            ('http://fluiddb.fluidinfo.com/one/two/',
                'http://fluiddb.fluidinfo.com/one/two/'),
            ('http://test.com/one/two/?referrer=http://a.b/c',
                'http://test.com/one/two/?referrer=http://a.b/c')
        )
        self.normalizeTest(expected, URI)



import sys
from books import ubook


coding = sys.getfilesystemencoding()
if not len(sys.argv) == 3:
    print "Form: python ubook.py 'Title of Book', 'Book Author'"
    sys.exit(1)

print ubook(sys.argv[1].decode(coding), sys.argv[2].decode(coding))







# -*- coding: utf-8 -*-

"""
    abouttag.location

    Standard about tags for URIs and URLs

    Copyright 2010 AUTHORS (see AUTHORS file)
    License: MIT, see LICENSE for more information
"""

from abouttag import about


def GEOnet(fli, fni, normalize=False, convention=u'geonet-1'):
    """Usage:
        from abouttag.uri import GEOnet

        normalURL = URI(-2601490, -3577649)
    """
    assert convention.lower() == u'geonet-1'
    return u'GEOnet%d_%d' % (fli, fni)


class TestGEOnet(about.AboutTestCase):
    def testFluidDBBadConvention(self):
        self.assertRaises(AssertionError, GEOnet, 1, 2, convention='unknown')

    def testFluidDBNormalize(self):
        expected = (
            ((1, 2), u'GEOnet1_2'),
            ((-99999, -77777), u'GEOnet-99999_-77777')
        )
        self.vectorTest(expected, GEOnet)



# -*- coding: utf-8 -*-

"""
    abouttag.music

    Standard about tags for tracks, albums and artists.

    Copyright 2010 AUTHORS (see AUTHORS file)
    License: MIT, see LICENSE for more information
"""

import unittest
from abouttag import about
from abouttag.nacolike import normalize
from abouttag.books import normalize_work

normalize_track = normalize_work(u'track', preserveAlpha=True,
                                           move_surname=False)
normalize_album = normalize_work(u'album', preserveAlpha=True,
                                           move_surname=False)


def track(title, artist, **kwargs):
    """Usage:
        from abouttag.music import track

        print track(u"Solilaï", u'Pierre Bensusan')

        print track(u'Bamboulé', u'Bensusan and Malherbe')

        print track(u'''Archie Campbell/Marjorie Campbell/Miss Lyall's '''
                    u'''Strathspey/Miss Lyall's Reel/The St Kilda Wedding''',
                    u'The Cast')

        track:solilaï (pierre bensusan)
        track:bamboulé (bensusan and malherbe)
        track:archie campbell marjorie campbell miss lyalls strathspey miss lyalls reel the st kilda wedding (the cast)
    """
    if 'convention' in kwargs:
        assert kwargs['convention'].lower() == u'track-u'

    return normalize_track(title, artist, **kwargs)


def album(title, artist, **kwargs):
    """Usage:
        from abouttag.music import album

        print album(u"Solilaï", u'Pierre Bensusan')
        album:solilaï (pierre bensusan)


        print album(u'Bamboulé', u'Bensusan and Malherbe')
        album:bamboulé (bensusan and malherbe)

        print album(u'''Archie Campbell/Marjorie Campbell/Miss Lyall's '''
                    u'''Strathspey/Miss Lyall's Reel/The St Kilda Wedding',
                    u'The Cast')

        album:archie campbell marjorie campbell miss lyalls strathspey miss lyalls reel the st kilda wedding (the cast)
    """
    if 'convention' in kwargs:
        assert kwargs['convention'].lower() == u'album-u'

    return normalize_album(title, artist, **kwargs)


def isrc_recording(isrc, **kwargs):
    """Usage:
        from aboutag.music import isrc_recording

        print isrc_recording(u'US-PR3-73-00012')
        isrc:US-PR3-73-00012

    """
    if 'convention' in kwargs:
        assert kwargs['convention'].lower() == u'isrc-1'

    isrc_stripped = isrc.replace(u'-', u'').upper()
    assert len(isrc_stripped) == 12
    return u'isrc:%s' % isrc_stripped


def artist(name, **kwargs):
    """Usage:
        from aboutag.music import artist

        print artist((u"John Renbourn")
        artist:john renbourn

        print artist(u"Crosby, Stills, Nash & Young")
        artist:crosby stills nash & young'

        print artist(u"Motörhead")
        artist:motörhead'
    """

    if 'convention' in kwargs:
        assert kwargs['convention'].lower() == u'artist-u'

    return 'artist:%s' % normalize(name, preserveAlpha=True)


class TestMusic(about.AboutTestCase):
    def testFluidDBNormalizeTrack(self):
        expected = (
            ((u"Solilaï", u'Pierre Bensusan'),
              u'track:solilaï (pierre bensusan)'),
            ((u"Down by the River", u'Crosby, Stills, Nash  &  Young'),
              u'track:down by the river (crosby stills nash & young)'),
            ((u'Bamboulé', u'Bensusan and Malherbe'),
             u'''track:bamboulé (bensusan and malherbe)'''),
            ((u'''Archie Campbell/Marjorie Campbell/Miss Lyall's '''
               '''Strathspey/Miss Lyall's Reel/The St Kilda Wedding''',
               u'The Cast'),
             u'''track:archie campbell marjorie campbell miss lyalls str'''
              '''athspey miss lyalls reel the st kilda wedding (the cast)''')
        )
        for (input, output) in expected:
            title, artists = input[0], input[1:]
            self.assertEqual((input, output),
                             (input, track(title, *artists)))

    def testFluidDBNormalizeAlbum(self):
        expected = (
            ((u"Solilaï", u'Pierre Bensusan'),
              u'album:solilaï (pierre bensusan)'),
            ((u"Déjà Vu", u'Crosby, Stills, Nash & Young'),
              u'album:déjà vu (crosby stills nash & young)'),
            ((u"Down by the River", u'Crosby, Stills, Nash  &  Young'),
              u'album:down by the river (crosby stills nash & young)'),
            ((u'Bamboulé', u'Bensusan and Malherbe'),
             u'''album:bamboulé (bensusan and malherbe)'''),
            ((u'''Archie Campbell/Marjorie Campbell/Miss Lyall's '''
               '''Strathspey/Miss Lyall's Reel/The St Kilda Wedding''',
               u'The Cast'),
             u'''album:archie campbell marjorie campbell miss lyalls str'''
              '''athspey miss lyalls reel the st kilda wedding (the cast)''')
        )
        for (input, output) in expected:
            title, artists = input[0], input[1:]
            self.assertEqual((input, output),
                             (input, album(title, *artists)))

    def testFluidDBNormalizeISRC(self):
        expected = (
            (u"US-PR3-73-00012", u'isrc:USPR37300012'),
            (u"USPR37300012", u'isrc:USPR37300012'),
            (u"us-pr3-73-00012", u'isrc:USPR37300012'),
            (u"uspr37300012", u'isrc:USPR37300012')
        )
        for (input, output) in expected:
            self.assertEqual((input, output),
                             (input, isrc_recording(input)))

    def testFluidDBNormalizeArtist(self):
        expected = (
            (u"John Renbourn", u'artist:john renbourn'),
            (u"The  Pentangle", u'artist:the pentangle'),
            (u"Pentangle, The", u'artist:pentangle the'),
            (u"Crosby, Stills, Nash & Young",
             u'artist:crosby stills nash & young'),
            (u"Crosby, Stills, Nash and Young",
             u'artist:crosby stills nash and young'),
            (u"Motörhead", u'artist:motörhead'),
        )
        for (input, output) in expected:
            self.assertEqual((input, output),
                             (input, artist(input)))


if __name__ == '__main__':
    unittest.main()
